#!/bin/bash
import re, os, argparse, datetime

#   IncludeFile = Bbl.rte/BShield.ini
includePat = re.compile(r'^\t?IncludeFile += +(?P<filename>[^\\/\n]+\.rte(?:[\\/][^\\/\n]+)+)(?://[^\n]*)?[ \t]*\n?')
numLines = 0
numImports = 0

## includeAction will be run with (included filename, match object)
## nonIncludeAction will be run with (line, isRoot)
def recurseFindIncludes(includeAction, nonIncludeAction, newfile, rootdir, root=None):
    if root is None:
        root = False
    with open(os.path.join(rootdir, newfile), 'r') as filestream:
        for line in filestream:
            mat = includePat.match(line)
            if mat is not None:
                filename = mat.group('filename').strip()
                includeAction(filename, mat)
                recurseFindIncludes(includeAction, nonIncludeAction, filename, rootdir)
            else:
                nonIncludeAction(line, root)

def recurseWriteFile(outstream, filename, rootdir):
    def foundInclude(includeName, *args):
        global numImports
        numImports += 1
        outstream.write('\n\t//CONCAT// Including from {}\n'.format(includeName))
    def foundLine(line, isRoot):
        global numLines
        numLines += 1
        if not isRoot and line != '\n':
            line = '\t'+line
        outstream.write(line)
    recurseFindIncludes(foundInclude, foundLine, filename, rootdir, root=True)

def recursePrintFile(filename, rootdir):
    def foundInclude(includeName, match):
        print(match.group('filename').strip())
    def foundLine(*args):
        pass
    recurseFindIncludes(foundInclude, foundLine, filename, rootdir, root=True)

if __name__ == '__main__':
    def good_index_file(filename):
        if not (os.path.isfile(filename) and os.path.basename(filename) == 'Index.ini'):
            raise Exception('Filename must exist and be named Index.ini; got {}'.format(filename))
        return filename
    def good_output_file(filename):
        if os.path.isfile(filename):
            raise Exception('Cannot replace existing file {}'.format(filename))
        return filename
    parser = argparse.ArgumentParser()
    parser.add_argument('filename', type=good_index_file, help='The file to start concatenating to; will be backed up')
    parser.add_argument('--output', type=good_output_file, default='Index.ini.new', help='Change the output; default %(default)s')
    parser.add_argument('--find-imports', action='store_true', help='Instead of writing anything, just find the include lines and print them out')

    args = parser.parse_args()

    rootdir = os.path.dirname(os.path.dirname(args.filename))
    modfile = os.path.relpath(args.filename, rootdir)
    if args.find_imports:
        recursePrintFile(modfile, rootdir)
    else:
        output = args.output
        if not os.path.isabs(args.output):
            output = os.path.join(os.path.dirname(args.filename), args.output)
        print('Writing to {}'.format(output))

        before = None
        after = None

        with open(output, 'w') as outfile:
            before = datetime.datetime.now()
            recurseWriteFile(outfile, modfile, rootdir)
            after = datetime.datetime.now()

        elapsed = after - before

        print('Successfully wrote file; traversed {} includes and concatenated {} lines in {}'.format(numImports, numLines, str(elapsed)))